package com.mortals.xhx.module.matter.service.impl;
import cn.hutool.core.io.FileUtil;
import cn.hutool.core.util.RandomUtil;
import com.mortals.framework.exception.AppException;
import com.mortals.xhx.common.utils.WordUtil;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import com.mortals.framework.service.impl.AbstractCRUDServiceImpl;
import com.mortals.xhx.module.matter.dao.MatterDatumFileDao;
import com.mortals.xhx.module.matter.model.MatterDatumFileEntity;
import com.mortals.xhx.module.matter.service.MatterDatumFileService;

import javax.naming.Context;
import java.io.File;
import java.io.FileNotFoundException;
import java.util.HashMap;
import java.util.Map;

/**
* MatterDatumFileService
* 材料附件 service实现
*
* @author zxfei
* @date 2022-01-12
*/
@Service("matterDatumFileService")
public class MatterDatumFileServiceImpl extends AbstractCRUDServiceImpl<MatterDatumFileDao, MatterDatumFileEntity, Long> implements MatterDatumFileService {

    @Value("${upload.path}")
    private String filePath;

    public static String word[] = {"doc", "dot", "wps", "wpt", "docx", "dotx", "docm", "dotm"};
    public static String imgs[] = {"jpg", "png", "jpeg", "gif"};
    public static String pdf[] = {"pdf"};

    @Override
    public void deleteFileByDatumId(Long datumId, Context context) {
        Map<String, Object> condition = new HashMap<>();
        condition.put("datumId", datumId);
        this.dao.delete(condition);
    }

    @Override
    protected void saveBefore(MatterDatumFileEntity entity, com.mortals.framework.model.Context context) throws AppException {
        this.validData(entity, context);
        String rootPath = this.filePath.endsWith("/") ? this.filePath : this.filePath + "/";
        String prePath = "file/preview/";
        String savePath = entity.getLocalFileUrl();
        String newName = genPreviewPath(rootPath, prePath,  savePath);
        entity.setPreviewUrl(prePath+newName);
    }

    protected void updateBefore(MatterDatumFileEntity entity, com.mortals.framework.model.Context context) throws AppException {
        this.validData(entity, context);
        String rootPath = this.filePath.endsWith("/") ? this.filePath : this.filePath + "/";
        String prePath = "file/preview/";
        String savePath = entity.getLocalFileUrl();
        String newName = genPreviewPath(rootPath, prePath,  savePath);
        entity.setPreviewUrl(prePath+newName);
    }

    private String genPreviewPath(String rootPath, String prePath, String tempPath) {
        //生成样表预览图片
        String samplePath = rootPath + tempPath;
        String filePath = rootPath + prePath;
        File pathDir = new File(filePath);
        if (!pathDir.exists()) {
            pathDir.mkdirs();
        }
        String newName = RandomUtil.randomNumbers(12) + ".jpg";
        String filePathAll = filePath + newName;
        //判断文件类型是否为doc pdf jpg png 等
        String extName = FileUtil.getSuffix(samplePath);
        if (this.isExsitArry(extName, word)) {
            WordUtil.convertWordToJPEG(samplePath, filePathAll);
        } else if (this.isExsitArry(extName, pdf)) {
            WordUtil.pdfToImages(samplePath, filePathAll);
        } else if (this.isExsitArry(extName, imgs)) {
            try {
                com.mortals.framework.util.FileUtil.copyFile(samplePath, filePathAll);
            } catch (FileNotFoundException e) {
                throw new RuntimeException(e);
            }
        }
        return newName;
    }

    public boolean isExsitArry(String chex, String arry[]) {
        for (String ex : arry) {
            if (chex.equalsIgnoreCase(ex)) {
                return true;
            }
        }
        return false;
    }
}