package com.mortals.xhx.module.matter.service.impl;

import com.mortals.framework.common.Rest;
import com.mortals.framework.exception.AppException;
import com.mortals.framework.model.Context;
import com.mortals.xhx.base.system.upload.service.UploadService;
import com.mortals.xhx.common.code.YesNoEnum;
import com.mortals.xhx.common.utils.BeanUtil;
import com.mortals.xhx.module.matter.model.*;
import com.mortals.xhx.module.matter.service.MatterDatumFileService;
import com.mortals.xhx.module.matter.service.MatterService;
import com.mortals.xhx.module.site.model.*;
import com.mortals.xhx.module.site.service.SiteDatumLibraryService;
import com.mortals.xhx.module.site.service.SiteService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import com.mortals.framework.service.impl.AbstractCRUDServiceImpl;
import com.mortals.xhx.module.matter.dao.MatterDatumDao;
import com.mortals.xhx.module.matter.service.MatterDatumService;
import org.springframework.util.ObjectUtils;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Date;
import java.util.List;
import java.util.stream.Collectors;

/**
 * MatterDatumService
 * 事项申请材料 service实现
 *
 * @author zxfei
 * @date 2022-01-12
 */
@Service("matterDatumService")
public class MatterDatumServiceImpl extends AbstractCRUDServiceImpl<MatterDatumDao, MatterDatumEntity, Long> implements MatterDatumService {

    @Autowired
    private MatterService matterService;
    @Autowired
    private MatterDatumFileService matterDatumFileService;
    @Autowired
    private UploadService uploadService;
    @Autowired
    private SiteDatumLibraryService siteDatumLibraryService;
    @Autowired
    private SiteService siteService;


    @Override
    protected void findAfter(MatterDatumEntity params, Context context, List<MatterDatumEntity> list) throws AppException {

        list.stream().peek(item->{
            List<MatterDatumFileEntity> matterDatumFileEntities = matterDatumFileService.find(new MatterDatumFileQuery().datumId(item.getId()));
            item.setDatumFileList(matterDatumFileEntities);
        }).count();
        super.findAfter(params, context, list);
    }

    @Override
    protected void saveBefore(MatterDatumEntity entity, Context context) throws AppException {
        if (!ObjectUtils.isEmpty(entity.getMatterId())) {
            MatterEntity matterEntity = matterService.get(entity.getMatterId());
            entity.setMatterName(matterEntity==null?"":matterEntity.getMatterName());
        }
        super.saveBefore(entity, context);
    }


    @Override
    protected void saveAfter(MatterDatumEntity entity, Context context) throws AppException {
        if (!ObjectUtils.isEmpty(entity.getMatterDatumFileList())) {
            entity.getMatterDatumFileList().stream().peek(item -> {
                item.setDatumId(entity.getId());
            }).count();
            matterDatumFileService.save(entity.getMatterDatumFileList());
        }
        super.saveAfter(entity, context);
    }

    @Override
    protected void updateAfter(MatterDatumEntity entity, Context context) throws AppException {
        if (!ObjectUtils.isEmpty(entity.getMatterDatumFileList())) {
            //先删除后新增
            Long[] fileIds = matterDatumFileService.find(new MatterDatumFileQuery().datumId(entity.getId())).stream()
                    .map(item -> {
                        uploadService.deleteFile(item.getFileUrl());
                        return item.getId();
                    }).toArray(Long[]::new);
            matterDatumFileService.remove(fileIds, context);
            entity.getMatterDatumFileList().stream().peek(item -> {
                item.setDatumId(entity.getId());
            }).count();
            matterDatumFileService.save(entity.getMatterDatumFileList());
        }
        super.updateAfter(entity, context);
    }


    @Override
    public Rest<String> addToLibrary(String datumIds, Long siteId, Context context) {
        if (ObjectUtils.isEmpty(siteId)) {
            throw new AppException("请选择对应站点");
        }
        List<Long> datumIdList = Arrays.asList(datumIds.split(",")).stream().map(Long::parseLong).collect(Collectors.toList());
        MatterDatumQuery datumQuery = new MatterDatumQuery();
        datumQuery.setIdList(datumIdList);
        List<MatterDatumEntity> entities = this.find(datumQuery);
        int success = 0;
        int fail = 0;
        List<SiteDatumLibraryEntity> addEntities = new ArrayList<>();

        for (MatterDatumEntity datum : entities) {
            SiteDatumLibraryEntity datumLibrary = updateOrSave(datum, siteId, context);
            //SiteMatterEntity siteMatterEntity = updateOrSave(matterEntity, siteId, context);
            if (!ObjectUtils.isEmpty(datumLibrary)) {
                addEntities.add(datumLibrary);
                success++;
            } else {
                fail++;
            }
        }
        if (!ObjectUtils.isEmpty(addEntities)) {
            siteDatumLibraryService.save(addEntities, context);
        }
        String msg = "当前加入材料已存在！";
        if (datumIdList.size() == 1) {
            if (success > 0) {
                msg = "加入材料成功！";
            }
        } else if (datumIdList.size() > 1) {
            if (success > 0 && fail == 0) {
                msg = String.format("加入材料成功%d条!", success);
            } else if (success > 0 && fail > 0) {
                msg = String.format("加入材料成功%d条，重复加入材料%d条！", success, fail);
            } else if (success == 0 && fail > 0) {
                msg = String.format("重复加入材料%d条！", fail);
            }
        }
        return Rest.ok(msg);
    }

    private SiteDatumLibraryEntity updateOrSave(MatterDatumEntity item, Long siteId, Context context) {
        SiteDatumLibraryEntity siteDatumLibraryEntity = siteDatumLibraryService.selectOne(new SiteDatumLibraryQuery().siteId(siteId).datumId(item.getId()));
        if (ObjectUtils.isEmpty(siteDatumLibraryEntity)) {
            SiteEntity siteEntity = siteService.getCache(siteId.toString());
            SiteDatumLibraryEntity datumLibraryEntity = BeanUtil.covert(item,SiteDatumLibraryEntity.class);
            datumLibraryEntity.setDatumId(item.getId());
            datumLibraryEntity.setSiteId(siteId);
            datumLibraryEntity.setSiteName(siteEntity==null?"":siteEntity.getSiteName());
            datumLibraryEntity.setCreateTime(new Date());
            if(context!=null&&context.getUser()!=null) {
                datumLibraryEntity.setCreateUserId(context.getUser().getId());
            }
            return datumLibraryEntity;
        }
        return null;
    }
}