package com.mortals.xhx.module.app.service.impl;

import cn.hutool.core.util.NumberUtil;
import cn.hutool.core.util.StrUtil;
import com.mortals.framework.model.PageInfo;
import com.mortals.framework.model.Result;
import com.mortals.xhx.common.code.YesNoEnum;
import com.mortals.xhx.common.key.Constant;
import com.mortals.xhx.module.app.model.*;
import com.mortals.xhx.module.app.service.AppInfoFieldService;
import com.mortals.xhx.module.app.service.AppService;
import com.mortals.xhx.module.app.service.AppVersionService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Bean;
import org.springframework.stereotype.Service;
import com.mortals.framework.service.impl.AbstractCRUDServiceImpl;
import com.mortals.framework.exception.AppException;
import com.mortals.framework.model.Context;
import com.mortals.xhx.module.app.dao.AppDatasetDao;
import com.mortals.xhx.module.app.service.AppDatasetService;
import org.springframework.util.ObjectUtils;

import java.util.Arrays;
import java.util.Date;
import java.util.List;

/**
 * AppDatasetService
 * 自助终端应用数据集 service实现
 *
 * @author zxfei
 * @date 2022-11-28
 */
@Service("appDatasetService")
public class AppDatasetServiceImpl extends AbstractCRUDServiceImpl<AppDatasetDao, AppDatasetEntity, Long> implements AppDatasetService {

    @Autowired
    private AppInfoFieldService appInfoFieldService;

    @Override
    protected void saveAfter(AppDatasetEntity entity, Context context) throws AppException {
        if (!ObjectUtils.isEmpty(entity.getAppInfoFieldList())) {
            entity.getAppInfoFieldList().stream().peek(item -> {
                item.setDatasetId(entity.getId());
                item.setCreateUser(context.getUser().getLoginName());
                item.setCreateUserId(context.getUser().getId());
                item.setCreateTime(new Date());
            }).count();
            appInfoFieldService.save(entity.getAppInfoFieldList());
        }

        super.saveAfter(entity, context);
    }

    @Override
    protected void updateAfter(AppDatasetEntity entity, Context context) throws AppException {
        if (!ObjectUtils.isEmpty(entity.getAppInfoFieldList())) {
            Long[] appInfoFieldIds = appInfoFieldService.find(new AppInfoFieldQuery().datasetId(entity.getId())).stream().map(AppInfoFieldEntity::getId).toArray(Long[]::new);
            appInfoFieldService.remove(appInfoFieldIds, context);
            entity.getAppInfoFieldList().stream().peek(item -> {
                item.setDatasetId(entity.getId());
                item.setCreateUser(context.getUser().getLoginName());
                item.setCreateUserId(context.getUser().getId());
                item.setCreateTime(new Date());
                item.setUpdateUser(context.getUser().getLoginName());
                item.setUpdateUserId(context.getUser().getId());
                item.setUpdateTime(new Date());
            }).count();
            appInfoFieldService.save(entity.getAppInfoFieldList());
        }

        super.updateAfter(entity, context);
    }

    @Override
    protected void removeAfter(Long[] ids, Context context, int result) throws AppException {
        List<AppInfoFieldEntity> appInfoFieldlist = appInfoFieldService.find(new AppInfoFieldQuery().datasetIdList(Arrays.asList(ids)));
        if (!ObjectUtils.isEmpty(appInfoFieldlist)) {
            Long[] idList = appInfoFieldlist.stream().map(item -> item.getId()).toArray(Long[]::new);

            appInfoFieldService.remove(idList, context);
        }
        super.removeAfter(ids, context, result);
    }

    @Override
    public Result<AppInfoFieldEntity> findCustomList(AppDatasetQuery appDatasetQuery, PageInfo pageInfo, Context context) throws AppException {
        return this.dao.getCustomList(appDatasetQuery, pageInfo);
    }
}